use std::collections::HashSet;
use csv::{Error, ByteRecord};

fn main() -> Result<(), Error> {
    let t0 = std::time::Instant::now();
    let set = read_hash_set("1000000 Sales Records.csv")?;
    println!("hash set created: {} {}", t0.elapsed().as_secs_f64(), set.capacity());
    let sorted_vec = read_vec("1000000 Sales Records.csv")?;
    println!("sorted_vector_created: {} {}", t0.elapsed().as_secs_f64(), sorted_vec.capacity());
    Ok(())
}

fn read_hash_set(file_name: &str) -> Result<HashSet<(Vec<u8>, Vec<u8>)>, Error> {
    let mut reader = csv::Reader::from_path(file_name)?;
    let mut set = HashSet::new();
    let mut record = csv::ByteRecord::new();
    while reader.read_byte_record(&mut record)? {
        set.insert((record[2].to_owned(), record[6].to_owned()));
    }
    Ok(set)
}

fn read_vec(file_name: &str) -> Result<Vec<csv::ByteRecord>, Error> {
    let mut reader = csv::Reader::from_path(file_name)?;
    let mut vec: Vec<ByteRecord> = reader.byte_records()
        .map(|r|r.unwrap())
        .collect();
    vec.sort_unstable_by(|a,b| a[0].cmp(&b[0]));
    Ok(vec)
}